package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import jdk.management.resource.internal.SimpleResourceContext;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.security.AccessControlContext;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.lang.Thread")
public final class ThreadRMHooks
{
    private long tid;
    
    public ThreadRMHooks() {
    }
    
    private static synchronized long nextThreadID() {
        return 0L;
    }
    
    @InstrumentationMethod
    private void init(ThreadGroup threadGroup, Runnable runnable, String s, long n, AccessControlContext accessControlContext, boolean b) {
        long nextThreadID = nextThreadID();
        ResourceIdImpl of = ResourceIdImpl.of(Long.valueOf(nextThreadID));
        ResourceRequest approver = ApproverGroup.THREAD_CREATED_GROUP.getApprover(this);
        long n2 = 1L;
        long request = 0L;
        try {
            request = approver.request(n2, of);
            if (request == 0L) {
                throw new ResourceRequestDeniedException("Resource limited: thread creation denied by resource manager");
            }
            this.init(threadGroup, runnable, s, n, accessControlContext, b);
            SimpleResourceContext simpleResourceContext = (SimpleResourceContext)SimpleResourceContext.getThreadContext(Thread.currentThread());
            simpleResourceContext.bindNewThreadContext((Thread)this);
        }
        finally {
            approver.request(-(request - n2), of);
        }
        this.tid = nextThreadID;
    }
    
    @InstrumentationMethod
    private void exit() {
        ResourceIdImpl of = ResourceIdImpl.of(Long.valueOf(this.tid));
        ResourceRequest approver = ApproverGroup.THREAD_CREATED_GROUP.getApprover(this);
        try {
            approver.request(-1L, of);
            SimpleResourceContext.removeThreadContext();
        }
        finally {
            this.exit();
        }
    }
}
